'use strict';

var options = require('minimist')(process.argv.slice(2));

if (!options.o) {
	console.error(' Enter output path');
	process.exit(1);
}

var conf = {
	outputPath: options.o,
	minify: options.min || options.minify || false,
	concat: options.conc || options.concat || true,
	stylusPaths: [ 'C:/GoogleDrive/Web/extra/stylus' ],
	srcPath: `${ options.o }source/`,
}



/* CONFG */
var

	lastFilesCount = -1,
	tasks = [],

	buildSystem = [
		// order: less > sass > stylus => css
		// css
		// { task: "less",    order: 1, path: `${ conf.srcPath }less/**/*.less` },
		// { task: "sass",    order: 2, path: `${ conf.srcPath }sass/**/*.s[c|a]ss` },
		{ task: "stylus",  order: 3, path: `${ conf.srcPath }stylus/**/*.styl` },
		{ task: "css",     order: 4, path: `${ conf.srcPath }css/**/*.css` },

		// js
		{ task: "coffee",  order: 5, path: `${ conf.srcPath }coffee/**/*.coffee` },
		{ task: "js",      order: 6, path: `${ conf.srcPath }js/**/*.js` },

		// html
		// { task: "ect",     order: 7, path: `${ conf.srcPath }ect/**/*.ect` },
		// { task: "ejs",     order: 8, path: `${ conf.srcPath }ejs/**/*.ejs` },
		// { task: "jade",    order: 9, path: `${ conf.srcPath }jade/**/*.jade` },
	];


/* Gulp */
var
		gulp 					= require("gulp"),
		minifyCSS 		= require("gulp-minify-css"),
		concatCSS 		= require("gulp-concat-css"),
		sass 					= require("gulp-sass"),
		stylus 				= require("gulp-stylus"),
		rename 				= require("gulp-rename"),
		uglify 				= require("gulp-uglify"),
		gulpConcat 		= require("gulp-concat"),
		less 					= require("gulp-less"),
		coffee 				= require("gulp-coffee"),
		postcss 			= require("gulp-postcss"),
		autoprefixer 	= require("autoprefixer"),
		changed 			= require("gulp-changed"),
		ejs 					= require("gulp-ejs"),
		ect 					= require("gulp-ect-simple"),
		jade 					= require("gulp-jade"),
		gulpif 				= require("gulp-if"),
		livereload 		= require("gulp-livereload"),
		nodeDir 			= require("node-dir"),
		_ 						= require("lodash");




function log(msg) {
	var str = "";

	if (arguments.length === 2)
		str = `  ${ msg } -> `;
	else
		str = `  ${ msg.name }: ${ msg.message } ${ msg.filename }`;

	console.log(str);
};


/* CSS */
gulp.task("sass", function() {
	log("SASS", true);

	gulp.src([ `${conf.srcPath}sass/**/*.s[c|a]ss`, `!${conf.srcPath}sass/**/_*.s[c|a]ss` ])
		.pipe(changed(`${conf.srcPath}css/`))
		.pipe(sass().on("error", sass.logError))
		.pipe(gulp.dest(`${conf.srcPath}css/_sass/`));
});

gulp.task("stylus", function() {
	log("Stylus", true);

	gulp.src([ `${conf.srcPath}stylus/**/*.styl`, `!${conf.srcPath}stylus/**/_*.styl` ])
		.pipe(changed(`${conf.srcPath}css/`))
		.pipe(stylus({ paths: conf.stylusPaths, compress: false }).on("error", log))
		.pipe(gulp.dest(`${conf.srcPath}css/_stylus/`));
});

gulp.task("less", function() {
	log("LESS", true);

	gulp.src([ `${conf.srcPath}less/**/*.less`, `!${conf.srcPath}less/**/_*.less` ])
		.pipe(changed(`${conf.srcPath}css/`))
		.pipe(less().on("error", log))
		.pipe(gulp.dest(`${conf.srcPath}css/_less/`));
});

gulp.task("css", function() {
	log("CSS", true);

	gulp.src([ `${conf.srcPath}css/**/*.css`, `!${conf.srcPath}css/**/_*.css` ])
		.pipe(changed(`${conf.outputPath}css/`))

		.pipe(gulpif(conf.concat, concatCSS('main.css', { rebaseUrls: false })))
		.pipe(gulpif(conf.minify, minifyCSS({ rebase: true, root: './' }).on("error", log)))

		.pipe(gulpif(conf.minify, rename(function(path) { path.basename += '.min' })))
		.pipe(gulpif(!conf.minify, rename({ dirname: '' })))

		.pipe(postcss([ autoprefixer({ browsers: [ "> 1%" ] }) ]))
		.pipe(gulp.dest(`${conf.outputPath}style/`))
		.pipe(livereload());
});


/* JavaScript */
gulp.task("js", function() {
	log("JavaScript", true);

	gulp.src([ `${conf.srcPath}js/**/*.js`, `!${conf.srcPath}js/**/_*.js` ])
		.pipe(changed(`${conf.outputPath}js/`))

		.pipe(gulpif(conf.concat, gulpConcat('main.js')))
		.pipe(gulpif(conf.minify, uglify().on("error", log)))

		.pipe(gulpif(conf.minify, rename(function(path) { path.basename += '.min' })))
		.pipe(gulpif(!conf.minify, rename({ dirname: '' })))

		.pipe(gulp.dest(`${conf.outputPath}js/`))
		.pipe(livereload());
});

gulp.task("coffee", function() {
	log("CoffeeScript", true);

	gulp.src([ `${conf.srcPath}coffee/**/*.coffee`, `!${conf.srcPath}coffee/**/_*.coffee` ])
		.pipe(changed(`${conf.srcPath}js/`))
		.pipe(coffee({ bare: true }).on('error', log))
		.pipe(gulp.dest(`${conf.srcPath}js/_coffee/`));
});


/* HTML */
gulp.task("ejs", function() {
	log("Ejs", true);

	gulp.src([ `${conf.srcPath}ejs/**/*.ejs`, `!${conf.srcPath}ejs/**/_*.ejs` ])
		.pipe(changed(`${conf.srcPath}html/`))
		.pipe(ejs().on("error", log))
		.pipe(gulp.dest(conf.outputPath))
		.pipe(livereload());
});

gulp.task("ect", function() {
	log("Ect", true);

	gulp.src([ `${conf.srcPath}ect/**/*.ect`, `!${conf.srcPath}ect/**/_*.ect` ])
		.pipe(changed(conf.outputPath))
		.pipe(rename({ dirname: '' }))
		.pipe(ect({
			options: {
				root: `${conf.srcPath}ect/`,
				ext: ".ect",
				open: "{{",
				close: "}}"
			}
		}).on("error", log))
		.pipe(gulp.dest(conf.outputPath))
		.pipe(livereload());
});

gulp.task("jade", function() {
	log("Jade", true);

	gulp.src([ `${conf.srcPath}jade/**/*.jade`, `!${conf.srcPath}jade/**/_*.jade` ])
		.pipe(changed(conf.outputPath))
		.pipe(rename({ dirname: '' }))
		.pipe(jade({
			pretty: true
		}).on("error", log))
		.pipe(gulp.dest(conf.outputPath))
		.pipe(livereload());
});


/* GULP */
gulp.task("watch", [ 'build' ], function() {
	livereload.listen();

	for (var i = 0, len = buildSystem.length; i < len; i++)
		tasks.push(gulp.watch(buildSystem[i].path, [ buildSystem[i].task ]));
});

gulp.task('build', function() {
	var tasks = _.sortBy(buildSystem, 'order');

	_.forOwn(tasks, function(task) {
		task.task && gulp.run(task.task);
	})
});

gulp.task('default', [ 'build' ]);


setInterval(function() {
	nodeDir.files(conf.srcPath, function(err, files) {
		if (err)
			return console.error(err);

		if (lastFilesCount !== -1 && lastFilesCount !== files.length) {
			console.log('New file added');

			tasks.forEach(function(t) {
				t.end();
			});

			tasks = [];

			gulp.run("watch");
		}

		lastFilesCount = files.length;
	});
}, 500);
